$(document).ready(function() {


    var App = {

        init: function() {
            // configs
            App.frontWords = SUGGEST_FRON_WORDS;
            App.backWords = SUGGEST_BACK_WORDS;

            App.delayOnError = 1000 * 60; // 1 minute

            App.delayBetweenRequests = 500;
            if(localStorage.delayBetweenRequests) {
                App.delayBetweenRequests = JSON.parse(localStorage.delayBetweenRequests);
            }
            $('#delay').val(App.delayBetweenRequests);

            // cache jquery
            App.$keyword = $('#keyword');
            App.$progress = $('#progress');
            App.$bar = App.$progress.find('.progress-bar');
            App.$results = $('#results');
			App.$progressText = $('#progress-text');
			App.$resultText = $('span#total-results');
            App.$log = $('#log');

            App.currentQueryNum = 0;
            App.variantsCount = 0;

            App.queryIndex = 0;
            App.keyword = '';
            App.isErrorRequest = false;
			App.stopScript = false;
			
            App.resultList = [];

            if(localStorage.keyword) {
                var keyword = JSON.parse(localStorage.keyword);
                App.$keyword.val(keyword);
            }

            $('#btn-search').click(App.clickBtnSearch);
			$('button.js-stop-btn').click(App.clickStopScript);			
			$('button.js-clear-btn').click(App.clickBtnClear);
			
            App.log('Start application');
        },

        clickBtnSearch: function(e) {
            e.preventDefault();

            App.delayBetweenRequests = 1 * $('#delay').val();
            localStorage.delayBetweenRequests = JSON.stringify(App.delayBetweenRequests);

            App.$results.val('');
            $('.js-copy-btn').removeAttr('disabled').text('Copy to clipboard');
			
            var mainKeyword = App.$keyword.val();
            localStorage.keyword = JSON.stringify(mainKeyword);

            App.keyword = mainKeyword;

            // prepare all queries:
            App.log('prepare all queries...');
            App.queries = [];
            for (var fw = 0; fw < App.frontWords.length; fw++) {
                for (var bw = 0; bw < App.backWords.length; bw++) {

                    var phrase = App.frontWords[fw] + ' ' + mainKeyword + ' ' + App.backWords[bw];

                    // stage 1
                    var query = {
                        q: phrase,
                        cp: null
                    };
                    App.queries.push(query);

                    // stage 2
                    var fullPhrase = phrase.replace(new RegExp(' ', 'g'), '%20');
                    var indexes = App.getIndicesOf('%20', fullPhrase);

                    for (var i = 0; i < indexes.length; i++) {
                        phrase = fullPhrase.slice(0, indexes[i]) + fullPhrase.slice(indexes[i] + 3);
                        //console.log(phrase);

                        var query = {
                            q: phrase,
                            cp: indexes[i]
                        };
                        App.queries.push(query);

                    }

                }
            }
            App.log('will be ' + App.queries.length + ' querie(s)');

            //console.log(App.queries);

            setTimeout(function() {
                App.askSuggestion();
            }, 1);
        },
		
		clickStopScript: function(e) {
			e.preventDefault();
			App.stopScript = true;
		},
		
		clickBtnClear: function(){
			if(localStorage.keyword){
				localStorage.keyword = null;	
			}
			if(localStorage.delayBetweenRequests) {
                localStorage.delayBetweenRequests = null;
            }
			window.location.reload();
		},
		
        askSuggestion: function() {
            if(App.queryIndex >= App.queries.length) {
                console.log('end');
                return;
            }

            var query = App.queries[App.queryIndex];

            var url = 'http://suggestqueries.google.com/complete/search?client=chrome&hl=en&gl=uk&q=' + query.q + '&callback=callback';
            if(query.cp) {
                url += '&cp=' + query.cp;
            }

            //console.log(url);
            //return;

            $.ajax({
                url: url,
                type: 'get',
                jsonp: "callback",
                dataType: "jsonp",
                success: function(response) {
                    //console.log(response);
                    if(!response[0] || !response[1]) {
                        App.log("ERROR, the response could not be processed correctly. pause for " + Math.round(App.delayOnError / 1000, 1) + ' second(s)');

                        App.$progressText.text(App.currentQueryNum + ' / ' + App.variantsCount + ' pause');
                        App.isErrorRequest = true;

                        setTimeout(function() {
                            App.askSuggestion();
                        }, App.delayOnError);
                        return;
                    }

                    var data = response[1];
                    var text = '';

                    for(var i = 0; i < data.length; i++) {
                        // remove duplicates
                        if(App.resultList.indexOf(data[i]) >= 0) {
                            continue;
                        }
                        App.resultList.push(data[i]);

                        text+= data[i] + '\r\n';
                    }

                    text = App.$results.val() + text;
                    App.$results.val(text);

                    App.currentQueryNum++;
                    App.updateProgressBar();

                    if(App.isErrorRequest) {
                        App.isErrorRequest = false;
                        App.delayBetweenRequests += 100;
                        App.log("Good response, timeout increased to " + App.delayBetweenRequests + 'ms');
                        $('#delay').val(App.delayBetweenRequests);
                        localStorage.delayBetweenRequests = JSON.stringify(App.delayBetweenRequests);
                    }

                    // check if this last query
                    App.queryIndex++;
                    if(App.queryIndex >= App.queries.length) {
                        App.$bar.removeClass('active');
                        App.log('got ' + App.resultList.length + ' unique results');
                        App.log('END');
                        return;
                    }
					
					if(!App.stopScript){
						setTimeout(function() {
							//console.log(query);
							App.askSuggestion();
						}, App.delayBetweenRequests + App.delayBetweenRequests * Math.random());
					}
                },
                error: function() {
                    App.log("ERROR, the response could not be processed correctly. pause for " + Math.Round(App.delayOnError / 1000, 1) + ' second(s)');

                    App.$progressText.text(App.currentQueryNum + ' / ' + App.variantsCount + ' pause');
                    App.isErrorRequest = true;

                    setTimeout(function() {
                        App.askSuggestion();
                    }, App.delayOnError);
                }

            });
        },

        updateProgressBar: function() {
            App.$progress.show();

            if(App.queryIndex >= App.queries.length) {
                App.$bar.removeClass('active');
            }
            else{
                App.$bar.addClass('active');
            }
			
			App.$resultText.text($('textarea.text-to-copy').val().split("\n").length + " Results");
			
			App.$progressText.text((App.queryIndex + 1) + ' / ' + App.queries.length);

            var width = Math.round((App.queryIndex + 1) / App.queries.length * 100);

            App.$bar.width(width + '%');
        },

        log: function(msg) {
            var txt = moment().format() + '> ' + msg + '\r\n';

            App.$log.append(txt);
        },

        getIndicesOf: function (searchStr, str) {
            var startIndex = 0, searchStrLen = searchStr.length;
            var index, indices = [];

            while ((index = str.indexOf(searchStr, startIndex)) > -1) {
                indices.push(index);
                startIndex = index + searchStrLen;
            }
            return indices;
        }

    };

    App.init();

});